--
-- AhwiM700
-- This is the specialization for automatic bale loaders - edit for Ahwi M700
--
-- @author  Manuel Leithner
-- @date  17/05/12
-- @edit  09/03/14 
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.


AhwiM700 = {};

function AhwiM700.prerequisitesPresent(specializations)
    return true;
end;

function AhwiM700:load(xmlFile)
	self.rotationNodes = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.rotationNodes.rotationNode(%d)", i);
        if not hasXMLProperty(xmlFile, key) then
            break;
        end;
        local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
        local rotSpeed = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, key.."#rotSpeed"), 1)*0.001);
        if node ~= nil then
            table.insert(self.rotationNodes, {node=node, rotSpeed=rotSpeed, rotation=0});
        end;
        i = i + 1;
    end;
	
	self.ptoDummy = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.powerTakeoffInput#dummy"));
	
	self.doUpdateCount = 50;
end;

function AhwiM700:delete()
end;

function AhwiM700:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AhwiM700:keyEvent(unicode, sym, modifier, isDown)
end;

function AhwiM700:update(dt)
	if self:getIsActive() then
        if self.isTurnedOn then
			for _, node in pairs(self.rotationNodes) do
				node.rotation = (node.rotation + dt*node.rotSpeed) % (2*math.pi);
				setRotation(node.node, node.rotation, 0, 0);
			end;
			
			self.doUpdateCount = 50;
		end;
    end;
	
	if self.doUpdateCount > 0 then
		for _, part in pairs(self.movingParts) do
			Cylindered.updateMovingPart(self, part);
		end;
		self.doUpdateCount = self.doUpdateCount - 1;
	end;	
end;

function AhwiM700:draw()
end

function AhwiM700:onDetach()
	setVisibility(self.ptoDummy, true);
end;

function AhwiM700:onAttach(attacherVehicle, jointDescIndex)
	if attacherVehicle.attacherJoints[jointDescIndex].ptoOutput ~= nil then
		setVisibility(self.ptoDummy, false);
	end;
end;